import asyncio
import time
from functools import wraps
from loguru import logger


def func_time_cost(step_name=None):
    """
    记录函数执行耗时，并支持指定步骤名称

    参数:
        step_name: 步骤名称，默认为函数名
    """

    def decorator(func):
        @wraps(func)  # 保留原函数元信息
        def wrapper(*args, **kwargs):
            nonlocal step_name
            step_name = step_name or func.__name__

            # 执行原函数并处理返回值
            result = None
            start_time = time.perf_counter()
            try:
                logger.debug(f"StepStart [{step_name}]")
                result = func(*args, **kwargs)

                # 检查是否为协程对象
                if asyncio.iscoroutine(result):

                    async def wrapped_coroutine():
                        try:
                            return await result
                        finally:
                            logger.debug(
                                f"StepTimeCost | [{step_name}][{time.perf_counter() - start_time:.6f}]秒"
                            )

                    return wrapped_coroutine()

                # 普通函数直接返回结果
                return result
            finally:
                # 普通函数在这里记录时间
                if not asyncio.iscoroutine(result):
                    logger.debug(
                        f"StepTimeCost | [{step_name}][{time.perf_counter() - start_time:.6f}]秒"
                    )

        return wrapper

    return decorator



import time
from typing import Optional

import aiohttp
from loguru import logger

from huibiao_framework.client.abstract_client import HuibiaoAbstractClient
from huibiao_framework.client.data_model.model_resp_vo import ModelBaseRespVo
from huibiao_framework.client.data_model.ocr.layout_detect_vo import DocumentLayoutDetectVo
from huibiao_framework.config.config import ModelConfig
from huibiao_framework.utils.image import ImageUtils


class LayoutDetectionClient(HuibiaoAbstractClient):
    """
    tender-ocr的文件实现类
    url: http://host:port/image_layout
    request:
        payload = {"data": encode_base64_string(file_path)}
        headers = {
                "Content-Type": "application/json",  # 根据需要设置内容类型
                "x-request-id": reqid
                }
    response:
        {
            "code": 0,
            "result": {
                "width": 1266,
                "height": 1806,
                "angle": 0,
                "version": "0.3.32",
                "layouts": [
                    {
                        "score": 0.983,
                        "type": "table",
                        "bbox": [
                                    [
                                        139,
                                        1128
                                    ],
                                    [
                                        1122,
                                        1128
                                    ],
                                    [
                                        1122,
                                        1618
                                    ],
                                    [
                                        139,
                                        1618
                                    ]
                                ]
                    },...
                ]
            }
        }
    """

    def __init__(self, session: Optional[aiohttp.ClientSession] = None):
        super().__init__(client_name="LayoutDetect", session=session, url=ModelConfig.LAYOUT_DETECTION_TYY_URL)

    async def query_by_image_base64(
        self, image_base64: str, reqid: str="", session_id: str = ""
    ) -> DocumentLayoutDetectVo:
        reqid = reqid if reqid is not None else ""
        session_id = session_id if session_id is not None else reqid

        session_tag = self.session_tag(session_id)

        payload = {"data": image_base64}
        headers = {
            "Content-Type": "application/json",
            "x-request-id": reqid,
        }
        # 发送异步POST请求
        start_time = time.time()
        try:
            async with self.session.post(
                self.url, json=payload, headers=headers
            ) as resp:
                sp_time = time.time() - start_time
                logger.debug(
                    f"{session_tag},resp-{resp.status},cost {sp_time:.2f}s"
                )
                resp.raise_for_status()
                response_data = await resp.json()
                response_data = ModelBaseRespVo[DocumentLayoutDetectVo](**response_data)
                return response_data.result
        except aiohttp.ClientError as e:
            logger.error(f"{session_tag} failed, {str(e)}")
            raise e

    async def query_by_image_path(self, image_path: str, reqid: str="", session_id: str = "") -> DocumentLayoutDetectVo:
        image_base64 = await ImageUtils.encode_base64(image_path)
        return await self.query_by_image_base64(image_base64=image_base64, reqid=reqid, session_id=session_id)



from pydantic import BaseModel, Field, field_validator
from typing import Generic, TypeVar, Optional, List
from pydantic_core.core_schema import FieldValidationInfo

from huibiao_framework.execption.model import (
    HuiZeModelResponseCodeError,
    HuiZeModelResponseFormatError,
)

T = TypeVar("T")


class ModelBaseRespVo(BaseModel, Generic[T]):

    # 响应状态码，0通常表示成功
    code: int = Field(..., description="响应状态码，0表示成功")

    # 响应消息
    message: str = Field("", description="响应状态描述信息")

    # 分析结果数据（泛型类型，可动态指定）
    result: Optional[T] = Field(None, description="分析结果数据，类型由泛型参数指定")

    @field_validator("code")
    def check_code_valid(cls, v: int) -> int:
        if v is None or v != 0:
            # 校验 code 合法性（通常 0 表示成功，非 0 表示错误）
            raise  HuiZeModelResponseCodeError(v)
        return v

    @field_validator("result")
    def check_result_consistent_with_code(
        cls, v: Optional[T], info: FieldValidationInfo
    ) -> Optional[T]:
        code = info.data.get("code")
        if code == 0 and v is None:
            # 整体后处理校验：code=0 时 result 必存在
            raise HuiZeModelResponseFormatError("Field 'result' is empty!")
        return v


from .execption import HuiBiaoException


class MinioClientExecution(HuiBiaoException):
    pass


class MinioClientConnectException(MinioClientExecution):
    def __init__(self, reason: str):
        self.reason = reason
        super().__init__(f"无法连接到Minio,原因：{reason}")


class MinioClientBucketNotExistsException(MinioClientExecution):
    def __init__(self, bucket_name: str):
        self.bucket_name = bucket_name
        super().__init__(f"桶 {bucket_name} 不存在")


class MinioClientOperationExecution(HuiBiaoException):
    pass


class MinioClientUploadException(MinioClientOperationExecution):
    def __init__(self, reason: str):
        self.reason = reason
        super().__init__(f"上传文件失败,原因：{reason}")


class MinioClientDownloadException(MinioClientOperationExecution):
    def __init__(self, reason: str):
        self.reason = reason
        super().__init__(f"下载文件失败,原因：{reason}")


class MinioClientListBucketsException(MinioClientOperationExecution):
    def __init__(self, reason: str):
        self.reason = reason
        super().__init__(f"列出桶失败,原因：{reason}")


class MinioClientRemoveObjectException(MinioClientOperationExecution):
    def __init__(self, reason: str):
        self.reason = reason
        super().__init__(f"删除对象失败,原因：{reason}")



from .huize_qwen32b_awq_client import HuiZeQwen32bQwqClient
from .minio_client import MinIOClient
from .embed_client import EmbedClient
from .ffs_progress_client import FfcsClient

__all__ = ["MinIOClient", "HuiZeQwen32bQwqClient", "EmbedClient", FfcsClient]



def frozen_attrs(*attrs):
    def decorator(cls):
        original_setattr = cls.__setattr__

        def new_setattr(self, name, value):
            if name in attrs and hasattr(self, name):
                raise AttributeError(
                    f"The attribute '{name}' is read-only and cannot be modified."
                )
            return original_setattr(self, name, value)

        cls.__setattr__ = new_setattr
        return cls

    return decorator



from .license import LimitResourceLicence
from .factory import LimitResourceLicenseFactory
from .licensee_decorator import (
    huize_qwen32b_awq_license,
    tender_image_ocr_license,
    tender_layout_detect_license,
)

__all__ = [
    "LimitResourceLicence",
    "LimitResourceLicenseFactory",
    "huize_qwen32b_awq_license",
    "tender_image_ocr_license",
    "tender_layout_detect_license",
]



import os
import time
from typing import Optional

import aiohttp
from loguru import logger

from huibiao_framework.client.abstract_client import HuibiaoAbstractClient
from huibiao_framework.config.config import ModelConfig
from huibiao_framework.execption.ocr import (
    DocumentParseResponseCodeError,
    DocumentParseResponseFormatError,
)


class DocumentParseClient(HuibiaoAbstractClient):
    """
    convert_to_pdf的文件实现类
    url: http://host:port/convert
    request:
        params = {
            "FileName": file_name,
            'Dpi': 144,           # 分辨率设置为144 dpi 200dpi，dpi越大精度越高
            'UseFormula': False,  # True/False 是否启用公式识别，启用会增加耗时
            'PdfPwd':'',          # pdf为加密密码
            'PageStart': page_start,       # 开始页码
            'PageCount': page_count,      # 设置解析页数
            'TableFlavor': 'html',  # html/md 表格内容格式 html 或 markdown
            'ParseMode': 'auto',  # auto/scan  设置解析模式为scan模式时会强制进行ocr
            'ImageUpload': False,
            'ImageParse': False,
            }
        headers = {
                    "x-request-id": reqid
                    }

    response:

    """

    def __init__(self, session: Optional[aiohttp.ClientSession]):
        super().__init__(client_name="ConvertToPdf", session=session, url=ModelConfig.DOCUMENT_PARSER_TYY_URL)

    async def convert_to_pdf(
        self,
        pdf_path: str,
        page_count: int,
        page_start: int,
        reqid: str,
        session_id: str = "",
    ) -> Optional[str]:

        session_tag = self.session_tag(session_id)

        start_time = time.time()
        try:
            file_name = os.path.basename(pdf_path)
            file = open(pdf_path, "rb")
            file_dict = {"File": file}
            params = {
                "FileName": file_name,
                "Dpi": 144,  # 分辨率设置为144 dpi 200dpi，dpi越大精度越高
                "UseFormula": False,  # True/False 是否启用公式识别，启用会增加耗时
                "PdfPwd": "",  # pdf为加密密码
                "PageStart": page_start,  # 开始页码
                "PageCount": page_count,  # 设置解析页数
                "TableFlavor": "html",  # html/md 表格内容格式 html 或 markdown
                "ParseMode": "auto",  # auto/scan  设置解析模式为scan模式时会强制进行ocr
                "ImageUpload": False,
                "ImageParse": False,
            }
            headers = {"x-request-id": reqid}
            # 发送异步POST请求
            async with self.__session.post(
                self.url,
                params=params,
                files=file_dict,
                headers=headers,
            ) as resp:
                sp_time = time.time() - start_time
                logger.debug(
                    f"{session_tag},resp-{resp.status}, 响应时间: {sp_time:.2f}秒"
                )
                resp.raise_for_status()  # 检查HTTP状态码
                response_code = resp.status
                response_data = await resp.json()  # 异步解析JSON
        except aiohttp.ClientError as e:
            logger.error(f"{session_tag}请求异常", e)
            raise e
        # 解析响应结果
        code: int = response_code
        if code != 200:
            logger.error(f"{session_tag}响应失败，code={code}")
            raise DocumentParseResponseCodeError(code)
        if "result" not in response_data:
            logger.error(
                f"{session_tag}响应格式异常, 缺少result字段"
            )
            raise DocumentParseResponseFormatError("响应体缺少result字段")

        return response_data["result"]



import time
from typing import Optional

import aiohttp
from loguru import logger

from huibiao_framework.client.abstract_client import HuibiaoAbstractClient
from huibiao_framework.client.data_model.model_resp_vo import ModelBaseRespVo
from huibiao_framework.client.data_model.vllm import (
    HuizeQwen32bAwqDto,
    HuizeQwen32bAwqVo,
)
from huibiao_framework.config.config import ModelConfig
from huibiao_framework.execption.model import HuiZeModelException


class HuiZeQwen32bQwqClient(HuibiaoAbstractClient):
    """
    慧泽Qwen-32B模型客户端
    url: http://vllm-qwen-32b.model.hhht.ctcdn.cn:9080/common/query
    request:
        {
        "Action": "NormalChat",
        "DoSample": true,
        "Messages": [
                {
                    "content": "请将下面这段英文翻译成中文：请将下面这段英文翻译成中文：I am a test。",
                    "role": "user"
                }
            ]
        }
    response:
        {
            "code": 0,
            "result": {
                "Output": "我是一个测试。",
                "TokenProbs": [
                    1.0
                ]
            },
            "message": "success"
        }
    """

    def __init__(self, session: Optional[aiohttp.ClientSession] = None):
        super().__init__(
            client_name="HuiZeQwen32bQwq", session=session, url=ModelConfig.REQUEST_URL
        )

    async def post(
        self, dto: HuizeQwen32bAwqDto, session_id: str = ""
    ) -> HuizeQwen32bAwqVo:
        session_tag = self.session_tag(session_id)
        logger.debug(
            f"{session_tag},contentLien={[len(i.content) for i in dto.Messages]}"
        )
        start_time = time.time()
        try:
            # 发送异步POST请求
            async with self.session.post(self.url, json=dto.model_dump()) as resp:
                sp_time = time.time() - start_time
                logger.debug(f"{session_tag},resp-{resp.status},cost {sp_time:.2f}s")
                resp.raise_for_status()
                response_data = await resp.json()
                response_data = ModelBaseRespVo[HuizeQwen32bAwqVo](**response_data)
                return response_data.result
        except HuiZeModelException as e:
            logger.error(f"{session_tag} resp error: {str(e)}")
            raise e
        except aiohttp.ClientError as e:
            logger.error(f"{session_tag} failed, {str(e)}")
            raise e

    async def query(self, content, session_id: str = "") -> HuizeQwen32bAwqVo:
        dto = HuizeQwen32bAwqDto()
        dto.Messages[0].content = content
        return await self.post(dto=dto, session_id=session_id)



from functools import wraps
from typing import Callable, Awaitable, TypeVar, Optional, ParamSpec, Union
from .factory import LimitResourceLicenseFactory
from .factory import DefaultLimitResourceName

P = ParamSpec("P")
T = TypeVar("T")


def _limit_resource_license_annotation(
    resource_type: DefaultLimitResourceName,
    user_info: Optional[str] = None,
) -> Callable[[Callable[P, Awaitable[T]]], Callable[P, Awaitable[T]]]:
    """
    资源限制许可装饰器，必须带参数使用：@_limit_resource_license_annotation(resource_type=...)

    :param resource_type: 资源类型（必填）
    :param user_info: 可选的用户信息，用于跟踪资源使用
    :return: 装饰器
    """

    def decorator(function: Callable[P, Awaitable[T]]) -> Callable[P, Awaitable[T]]:
        @wraps(function)
        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
            lic = LimitResourceLicenseFactory.genLicense(
                resource_name=resource_type, user_info=user_info
            )
            async with lic:
                return await function(*args, **kwargs)

        return wrapper

    return decorator


def _predefined_license(resource_type: DefaultLimitResourceName):
    """创建支持user_info参数和无括号使用的预定义装饰器"""

    def decorator_or_factory(
        user_info: Optional[Union[str, Callable[P, Awaitable[T]]]] = None,
    ) -> Union[
        Callable[P, Awaitable[T]],
        Callable[[Callable[P, Awaitable[T]]], Callable[P, Awaitable[T]]],
    ]:
        # 处理无参数使用的情况：@huize_qwen32b_awq_license
        if callable(user_info):
            func = user_info
            return _limit_resource_license_annotation(resource_type=resource_type)(func)

        # 处理带user_info参数的情况：@huize_qwen32b_awq_license(user_info="xxx")
        def wrapper(f: Callable[P, Awaitable[T]]) -> Callable[P, Awaitable[T]]:
            return _limit_resource_license_annotation(
                resource_type=resource_type,
                user_info=user_info,  # 可能为None
            )(f)

        return wrapper  # 一个装饰器函数，接收一个异步函数，返回一个新的异步函数，参数和返回类型不变

    return decorator_or_factory


# 预定义Qwen32B AWQ模型的资源许可装饰器
# 支持: @huize_qwen32b_awq_license 或 @huize_qwen32b_awq_license(user_info="xxx")
huize_qwen32b_awq_license = _predefined_license(
    resource_type=DefaultLimitResourceName.HUIZE_QWEN_32B_AWQ
)

# 预定义 tender_image_ocr 资源许可装饰器
tender_image_ocr_license = _predefined_license(
    resource_type=DefaultLimitResourceName.TENDER_IMAGE_OCR
)

# 预定义 tender_layout_detect 模型资源许可装饰器
tender_layout_detect_license = _predefined_license(
    resource_type=DefaultLimitResourceName.TENDER_LAYOUT_DETECT
)



from .result import Result, BatchResult, SingleResult, FileOperator
from .task_result import TaskResult

__all__ = [
    "FileOperator",
    "TaskResult",
    "Result",
    "SingleResult",
    "BatchResult",
]



import os
import time
from abc import ABC, abstractmethod
from typing import Generic, List, TypeVar, final, Dict, Type

from huibiao_framework.execption.result_execption import ResultAlreadyFinishedException
from huibiao_framework.utils.meta_class import ConstantClass


class ResultStatusTagConstant(ConstantClass):
    DONE = "__DONE"


D = TypeVar("D")


# region 文件操作抽象类
class FileOperator(Generic[D], ABC):
    """
    文件操作抽象类
    """

    @classmethod
    @abstractmethod
    def file_suffix(cls) -> str:
        """子类必须实现此类方法，对丁文件的后缀名，不能包含点"""
        pass

    @classmethod
    @abstractmethod
    def load(cls, path: str, **kwargs) -> D:
        """
        从本地加载文件
        """
        pass

    @classmethod
    @abstractmethod
    def save(cls, data: D, path, **kwargs):
        """
        保存文件到本地
        """
        pass


# endregion

F = TypeVar("F", bound=FileOperator)


# region 抽象结果类
class Result(Generic[F, D], ABC):
    def __init__(self, operator: Type[F]):
        self.__operator: Type[F] = operator

    @abstractmethod
    def is_completed(self, *args, **kwargs) -> bool:
        """
        该资源是否准备完毕
        """
        pass

    @abstractmethod
    def complete(self):
        pass

    @property
    def operator(self) -> F:
        return self.__operator

    @abstractmethod
    def load(self, **kwargs):
        pass

    @abstractmethod
    def save(self, **kwargs):
        pass

    @abstractmethod
    def description(self) -> str:
        pass

    def __repr__(self):
        return self.description()

    def __str__(self):
        return self.description()


# endregion


# region 单个文件结果类
class SingleResult(Result[F, D]):
    def __init__(self, path: str, operator: Type[F] = None):
        super().__init__(operator)
        self.__path = path
        self.__data = None

    @property
    def data(self) -> D:
        return self.__data

    @final
    def is_completed(self) -> bool:
        return os.path.exists(self.__path)

    @final
    def complete(self):
        if not self.is_completed():
            self.save()

    def load(self, **kwargs):
        if os.path.exists(self.__path):
            self.__data = self.operator.load(path=self.__path, **kwargs)

    def save(self, **kwargs):
        if self.__data is not None:
            os.makedirs(os.path.dirname(self.__path), exist_ok=True)
            self.operator.save(path=self.__path, data=self.__data, **kwargs)

    @property
    def path(self) -> str:
        return self.__path

    def description(self) -> str:
        return f"{self.operator.__class__.__name__}[{self.operator.path}]"


# endregion


# region 多个文件结果类
class BatchResult(Result[F, D]):
    def __init__(self, folder: str, operator: Type[F]):
        super().__init__(operator)
        self.__folder = folder
        self.__data: Dict[int, D] = {}
        os.makedirs(self.folder, exist_ok=True)
        self.complete_tag_path = os.path.join(
            self.folder, ResultStatusTagConstant.DONE
        )


    def __getitem__(self, idx) -> F:
        return self.__data[idx]

    def __len__(self):
        return len(self.__data)

    @property
    def folder(self) -> str:
        return self.__folder

    def load(self, **kwargs):
        for path in self.path:
            idx = int(os.path.basename(path).split(".")[0])
            self.__data[idx] = self.operator.load(path=path)

    def save(self, **kwargs):
        for idx, data in self.__data.items():
            self.operator.save(
                path=self.genPath(idx),
                data=data,
                **kwargs,
            )

    def genPath(self, idx: int):
        return os.path.join(self.folder, f"{str(idx)}.{self.operator.file_suffix()}")

    @final
    def is_completed(self) -> bool:
        return os.path.exists(self.complete_tag_path)

    def append(self, data: D):
        if self.is_completed():
            raise ResultAlreadyFinishedException(
                folder=self.folder
            )
        idx = len(self) + 1
        self.__data[idx] = data

    @final
    def complete(self):
        if os.path.exists(self.complete_tag_path):
            return  # 忽略该操作
        with open(
            self.complete_tag_path,
            "w",
        ) as f:
            local_time = time.localtime(time.time())
            formatted_time = time.strftime("%Y-%m-%d %H:%M:%S", local_time)
            f.write(formatted_time)  # 往目录下写入一个文件，包含当前时间

    @property
    def path(self) -> List[str]:
        return [
            os.path.join(self.folder, r)
            for r in os.listdir(self.folder)
            if r != ResultStatusTagConstant.DONE
        ]

    def description(self) -> str:
        return f"{self.operator.__class__.__name__}[{self.operator.folder}]"


# endregion
